-- | Aspects of CCG that interface with meaning representations.
module ELCS.CCG.Semantics (
  semToSynType

  -- * References
  -- $references
) where

import ELCS.CCG.CCG
import ELCS.MR.Type
import ELCS.Util

import Control.Exception

-- | Composes a syntactic category that directly corresponds to a given semantic type.
--   The rules for this are simple: The 'Truth' type corresponds to the category "S" and
--   the 'Entity' type to "NP". 'Arrow's in the type are replaced by vertical slashes.
--   This scheme follows [Kwiatkowski et al. (2011)] as well as the implementation in [SPF].
semToSynType :: Type -> SynCat
semToSynType t = assert (isCompleteType t) $
  case t of
    -- Note the reordering because the result is on the left side with slashes
    Arrow t1 t2 -> VerticalSlash (semToSynType t2) (semToSynType t1)
    Entity      -> Atom "NP" Nothing
    Truth       -> Atom "S" Nothing
    Var _       -> throw $ FatalException "Type is not complete"
    
{-$references
  * [Kwiatkowski et al. (2011)] Kwiatkowski, Tom, Luke Zettlemoyer, Sharon Goldwater, and Mark Steedman. ‘Lexical Generalization in CCG Grammar Induction for Semantic Parsing’. In Proceedings of the Conference on Empirical Methods in Natural Language Processing, 1512–1523, 2011. http://dl.acm.org/citation.cfm?id=2145593.
  * [SPF] https://bitbucket.org/yoavartzi/spf
-}