module ELCS.DRT.Alignment (
  removeInnerAlignments,
  BinaryAlignment(..),
  SplitDirection(..),
  SplitType(..),
  evalAlignment
) where

import ELCS.MR.Label
import ELCS.DRT.AST

-- | Removes all alignments except those on Simple conditions to simulate an alignment tool.
removeInnerAlignments :: DRT -> DRT
removeInnerAlignments e =
  case drtHead e of
    Cond (SimpleHead _ _) -> e
    _                     -> e { drtLabel = (drtLabel e) { mrLabelAlignment = Just [] },
                                 drtSubs = map removeInnerAlignments (drtSubs e) }
  
data BinaryAlignment = Head | Complement deriving (Show,Eq)

data SplitDirection = ForwardSplit | BackwardSplit deriving (Show,Eq)
data SplitType = SplitType { splitDirection  :: SplitDirection
                           , splitPoint      :: Int }
                           deriving (Eq,Show)

evalAlignment :: SplitType -> [Int] -> Maybe BinaryAlignment
evalAlignment s al = case al of
                       []                               -> Nothing
                       _  | all (isHeadEdge s) al       -> Just Head
                          | all (isComplementEdge s) al -> Just Complement
                          | otherwise                   -> Nothing

isHeadEdge :: SplitType -> Int -> Bool
isHeadEdge s i = case splitDirection s of
                   ForwardSplit  -> i < (splitPoint s)
                   BackwardSplit -> i >= (splitPoint s)
                
isComplementEdge :: SplitType -> Int -> Bool
isComplementEdge s = not . (isHeadEdge s)


