module ELCS.GMB (
  Instance(..)
, initialise
, readFile
) where

import Prelude hiding (readFile)
import System.FilePath
import Data.Maybe
import System.IO hiding (readFile)
import qualified Data.Text as Txt
import qualified Data.Text.IO as IO
import Data.Text.Encoding
import qualified Data.ByteString.Lazy as BS
import qualified Codec.Archive.Zip as Zip
--import Debug.Trace


data Instance = FileInstance { path :: FilePath }
              | ZipInstance { path :: FilePath }
                deriving (Eq, Show)
                   
initialise :: FilePath -> Instance
initialise p =
  case (takeExtension p) of
    ".zip" -> ZipInstance p
    _      -> FileInstance p

readFile :: Instance -> FilePath -> IO String
readFile gmb p = {-traceIO ("Loading file " ++ show p) >>-} readFile' gmb p

readFile' :: Instance -> FilePath -> IO String
readFile' (FileInstance p) p' = IO.readFile (p </> p') >>= return . Txt.unpack
readFile' (ZipInstance p) p' =
  do
    file <- openBinaryFile p ReadMode
    compressed <- BS.hGetContents file
    let a = Zip.toArchive compressed
    let entry = fromJust $ Zip.findEntryByPath p' a -- todo error handling
    let cnt = BS.toStrict $ Zip.fromEntry entry
    hClose file
    return $ Txt.unpack $ decodeUtf8 cnt
