{-# LANGUAGE BangPatterns, OverloadedStrings #-}
{-|
  Handles reading of GMB metadata.
-}
module ELCS.GMB.Metadata (
  Metadata
, readMetadata
, getSubCorpus
) where

import qualified ELCS.GMB as GMB
import ELCS.GMB.Document

import System.FilePath

-- | Metadata information is represented simply as a key-value pair.
type Metadata = [(String, String)]

-- | Loads the metadata for the given document.
readMetadata :: GMB.Instance -> DocumentId -> IO Metadata
readMetadata gmb doc = do
  content <- GMB.readFile gmb (docPath doc </> "en.met")
  let mdlines = filter (not . null) $ lines content
  return $ map parseKeyValuePair mdlines
  
parseKeyValuePair :: String -> (String, String)
parseKeyValuePair s = let
    (k,vs) = break (== ':') s
    -- Drop the colon and any following whitespace
    v = dropWhile (`elem` [' ','\160']) $ drop 1 vs
  in (k,v)
  
-- | Filters a list of documents by subcorpus name.
getSubCorpus :: String -> GMB.Instance -> [DocumentId] -> IO [DocumentId]
getSubCorpus n gmb docs = do
  mds <- mapM (readMetadata gmb) docs
  let mdDocs = zip docs mds
  let filtered = filter (\(_,md) -> lookup "subcorpus" md == Just n) mdDocs
  return $ map fst filtered