{-# LANGUAGE DeriveDataTypeable, DeriveGeneric, DefaultSignatures #-}

module ELCS.MR.Type(
  Type(..)
, isCompleteType
, arrowCount
) where

import Control.DeepSeq.Generics
import GHC.Generics(Generic)

-- | A simple typing scheme for meaning representations.
data Type = Entity
          | Truth
          | Var Int
          | Arrow Type Type
          deriving (Show, Eq, Ord, Read, Generic)    
  
instance NFData Type where rnf = genericRnf

            
-- | A type is complete if it contains no variables.
isCompleteType :: Type -> Bool
isCompleteType t = case t of
  Var _       -> False
  Arrow t1 t2 -> isCompleteType t1 && isCompleteType t2
  _           -> True
  
-- | Counts the arrows in a type. This provides a simple measure for complexity.
arrowCount :: Type -> Int
arrowCount t = case t of
  Arrow t1 t2 -> 1 + (arrowCount t1) + (arrowCount t2)
  _           -> 0