{-| The main module for the Collate command.
-}
module ELCS.Main.Collate (
  mainWith
) where

import ELCS.Main.Common.CLOptions
import ELCS.CCG
import ELCS.CCG.Serialise.Collate
import ELCS.Util

import Data.List
import Control.Monad.Reader
import System.FilePath.Glob
import Control.Exception

mainWith :: CollateOptions -> IO ()
mainWith opts = runReaderT collateMain opts

-- | Main function for the collate command.
collateMain :: ReaderT CollateOptions IO ()
collateMain = do
  opts <- ask
  files <- liftIO $ expandGlobs (coll_globs opts)
  let inOpts = processInputOptions opts
  let outOpts = processOutputOptions opts
  liftIO $ collateDRT inOpts outOpts files (coll_output opts)
  
-- | Extracts the appropriate serialisation options from the command line argument.
--   Errors out on invalid input.
processInputOptions :: CollateOptions -> SerialisationOptions
processInputOptions opts =
  defaultSerialisationOptions
    { serCodec = case coll_inputCodec opts of
                   "JSON" -> JSON
                   c -> throw $ FatalException ("Invalid input codec: " ++ c)
    , serWithStats = coll_inputWithStats opts
    }
  
-- | Extracts the appropriate serialisation options from the command line argument.
--   Errors out on invalid input.
processOutputOptions :: CollateOptions -> SerialisationOptions
processOutputOptions opts =
  defaultSerialisationOptions
    { serWithStats = coll_withStats opts
    }
  
-- | Returns the sorted union of all globbed files.
expandGlobs :: [String]      -- ^ A list of globs to expand.
            -> IO [FilePath] -- ^ The sorted list of files matched by the globs.
expandGlobs gs = do
  expanded <- mapM glob gs
  return (sort $ concat expanded)
