{-# LANGUAGE BangPatterns, DeriveDataTypeable #-}
module ELCS.Util (
  FatalException(..)
, tryFatalException
, assertMsg
, containsDuplicatesOrd
, mapSnd
, mapSndM
, chunksOf
, time
) where

--import Control.Exception(assert)
import Debug.Trace
import qualified Data.Set as Set
import Data.Time.Clock.POSIX (getPOSIXTime)
import Control.Exception
import Data.Typeable

data FatalException = FatalException String
  deriving (Show, Typeable)

instance Exception FatalException where

-- | Convenience variant of try for catching FatalException
tryFatalException :: IO a -> IO (Either FatalException a)
tryFatalException = try

-- | Functions like assert, but traces a message if the assertion fails.
assertMsg :: String -> Bool -> a -> a
assertMsg msg cond r = assert (if cond then True else trace msg False) r
--assertMsg _ = assert

containsDuplicatesOrd :: (Eq a, Ord a) => [a] -> Bool
containsDuplicatesOrd xs = containsDuplicatesOrdHelper xs Set.empty

-- todo: how to write this without direct recursion, but while keeping early termination?
containsDuplicatesOrdHelper :: (Eq a, Ord a) => [a] -> Set.Set a -> Bool
containsDuplicatesOrdHelper (x:xs) seen =
  if x `Set.member` seen
    then True
    else containsDuplicatesOrdHelper xs (Set.insert x seen)
containsDuplicatesOrdHelper [] _ = False

mapSnd :: (b -> c) -> [(a,b)] -> [(a,c)]
mapSnd f = map (\(x,y) -> (x,f y))

mapSndM :: (Monad m) => (b -> m c) -> [(a,b)] -> m [(a,c)]
mapSndM f = mapM helper
  where helper (x,y) = do z <- f y
                          return (x,z)
                          
-- | Chops a list into chunks of a given size.
chunksOf :: Int -> [a] -> [[a]]
chunksOf _ [] = []
chunksOf s l = (take s l):(chunksOf s (drop s l))

getTime :: IO Double
getTime = realToFrac `fmap` getPOSIXTime

-- | Measure the (wall-clock) time it takes for an action to complete.
time :: IO a -> IO (Double, a)
time act = do
  start <- getTime
  result <- act
  end <- getTime
  let !delta = end - start
  return (delta, result)

